@@ -10,6 +10,12 @@
 #' @exportClass ThingsboardApi
 #' @importFrom methods new
 #'
+#' @seealso The `ThingsboardApi` class methods :
+#' - [ThingsboardApi_checkToken] for checking and refreshing the token
+#' - [ThingsboardApi_getToken] for getting authorisation token from Thingsboard server for a specific device
+#' - [ThingsboardApi_getKeys] for fetching data keys of an entity
+#' - [ThingsboardApi_getValues] and [ThingsboardApi_getTelemetry] for fetching telemetry of an entity
+#'
 #' @examples
 #' \donttest{
 #' thinksboard_api = tryCatch(
@@ -45,6 +51,10 @@ ThingsboardApi <- setRefClass(
 
 #' Check if the token is timeouted and refresh it if necessary
 #'
+#' @description
+#' This method is automatically call by each other methods of the class [ThingsboardApi].
+#' So, except for debugging purpose, it's not useful to call directly.
+#'
 #' @name ThingsboardApi_checkToken
 #' @return [NULL]
 #'
@@ -60,6 +70,10 @@ ThingsboardApi$methods(
 
 #' Get authorisation token from thingsboard server for a specific device
 #'
+#' @description
+#' This method is automatically called by [ThingsboardApi_checkToken], and so by any other methods
+#' of the class [ThingsboardApi] as needed. Except for debugging purpose, it's not useful to call directly.
+#'
 #' @name ThingsboardApi_getToken
 #' @param timeOut [numeric] number of second before token timeout (default field `tokenTimeOut`)
 #'
@@ -93,6 +107,7 @@ ThingsboardApi$methods(
 #' Fetch data keys for an entity
 #'
 #' @name ThingsboardApi_getKeys
+#'
 #' @details
 #' The description of this operation in API documentation is here: <https://thingsboard.io/docs/user-guide/telemetry/#get-telemetry-keys>
 #'
@@ -130,27 +145,12 @@ ThingsboardApi$methods(
 )
 
 
-#' Fetch values from an entity
+#' Fetch telemetry
 #'
 #' @description
-#' See: <https://thingsboard.io/docs/user-guide/telemetry/#get-telemetry-values>
-#'
-#' This method has a strong limitation as the 'ThingsBoard' API only send the
-#' 100 last values of each key.
-#' Use the method getTelemetry to override this limitation.
+#' See [ThingsboardApi_getTelemetry].
 #'
 #' @name ThingsboardApi_getValues
-#' @param entityId A [character] with the entity ID given (See <https://thingsboard.io/docs/user-guide/entity-views/>)
-#' @param keys Vector of [character] with the list of keys from which getting the telemetry values
-#' @param entityType A [character] (default "DEVICE")
-#' @param startTs A [numeric] or a [POSIXct] representing respectively the epoch or the date of the start of data extraction period
-#' @param endTs A [numeric] or a [POSIXct] representing respectively the epoch or the date of the end of data extraction period
-#'
-#' @return A [data.frame] with one row per data and 3 columns:
-#'   `key`: A [character] with the key,
-#'   `ts`: A [POSIXct] with the timestamp of the data,
-#'   `value`: A [numeric] with the value of the data
-#'
 #'
 NULL
 ThingsboardApi$methods(
@@ -200,7 +200,8 @@ ThingsboardApi$methods(
     lV <- lapply(names(lV),
                  function(x) {
                    df <- data.frame(key = x,
-                                    v = matrix(unlist(lV[[x]]), ncol = 2, byrow = TRUE))
+                                    v = matrix(unlist(lV[[x]]), ncol = 2, byrow = TRUE),
+                                    stringsAsFactors = FALSE)
                    colnames(df) <-
                      c("key", "ts", "value")
                    df
@@ -212,7 +213,8 @@ ThingsboardApi$methods(
     } else {
       dfV <- data.frame(key = character(),
                         ts = as.POSIXct(character()),
-                        value = numeric())
+                        value = numeric(),
+                        stringsAsFactors = FALSE)
     }
     return(dfV)
   }
@@ -222,13 +224,22 @@ ThingsboardApi$methods(
 #' Fetch telemetry
 #'
 #' @description
+#' Fetch telemetry data of an entity.
 #'
-#' Fetch telemetry data for an entity. See [ThingsboardApi_getValues] for the arguments.
+#' It uses the following API: <https://thingsboard.io/docs/user-guide/telemetry/#get-telemetry-values>
+#'
+#' The method `getValues` has a strong limitation as the 'ThingsBoard' API only send the
+#' 100 last values of each key. The method `getTelemetry` overcomes this limitation by
+#' automatically by calling `getValues` in a loop.
 #'
-#' See: <https://thingsboard.io/docs/user-guide/telemetry/#get-telemetry-values>
 #'
 #' @name ThingsboardApi_getTelemetry
-#' @param ... Parameters passed through method [ThingsboardApi_getValues]
+#' @rdname ThingsboardApi_getTelemetry
+#'
+#' @param entityId A [character] with the entity ID given (See <https://thingsboard.io/docs/user-guide/entity-views/>)
+#' @param keys Vector of [character] with the list of keys from which getting the telemetry values
+#' @param entityType A [character] (default "DEVICE")
+#' @param startTs A [numeric] or a [POSIXct] representing respectively the epoch or the date of the start of data extraction period
 #' @param endTs A [numeric] or a [POSIXct] representing respectively the epoch or the date of the end of data extraction period
 #'
 #' @return A [data.frame] with one row per data and 3 columns:
@@ -242,19 +253,16 @@ ThingsboardApi$methods(
 NULL
 ThingsboardApi$methods(
   getTelemetry = function(..., endTs) {
-    the_end <- FALSE
-    df <- NULL
-    while(!the_end) {
-      dfI <- getValues(..., endTs = endTs)
-      the_end <- nrow(dfI) == 0
-      if(is.null(df)) {
-        df <- dfI
-      } else {
-        df <- rbind(df, dfI)
-      }
-      df_minTs <- df %>% arrange(ts) %>% group_by(key) %>% slice(1)
-      endTs <- max(df_minTs$ts)
+    l <- list()
+    i <- 0
+    while (TRUE) {
+      i <- i + 1
+      l[[i]] <- getValues(..., endTs = endTs)
+      if (nrow(l[[i]]) == 0) break
+      df_minTs <- l[[i]] %>% arrange(ts) %>% group_by(key) %>% slice(1)
+      endTs <- max(df_minTs$ts) - 1
     }
+    df <- do.call(rbind, l)
     df <- unique(df)
     return(df)
   }
